# Set up the toolchain. For out proposes, we'll use clang as CC, CXX and LD for all
# targets, but this can be easily modified.

CC := clang
CXX := $(CC)
LD := $(CC)

# CFLAGS   - Flags common to C and C++
# CCFLAGS  - Extra C exclusive flags
# CXXFLAGS - Extra C++ exclusive flags
# CCFLAGS_ios / CXXFLAGS_ios  - Effective flags used when compiling C/++ for ios
# CCFLAGS_mac / CXXFLAGS_mac  - Effective flags used when compiling C/++ for android 

CFLAGS :=
ifeq ($(DEBUG),true)
CFLAGS += -g
endif
CFLAGS += -Wall -Wextra
CFLAGS += -Werror
CFLAGS += -Winvalid-noreturn
CFLAGS += -Wshadow
CFLAGS += -Wpointer-arith
CFLAGS += -Wzero-length-array
CFLAGS += -Wdeprecated
CFLAGS += -Wdeprecated-declarations
CFLAGS += -Wno-unused-parameter
CFLAGS += -Wno-missing-field-initializers
CFLAGS += -Wno-c11-extensions
CFLAGS += -Wno-newline-eof
CFLAGS += -Wno-unknown-warning-option
CFLAGS += -Wunreachable-code -Wunreachable-code-return -Wunreachable-code-break
CFLAGS += -Widiomatic-parentheses
CFLAGS += -Wheader-hygiene
CFLAGS += -Wshadow-field-in-constructor
CFLAGS += -Wswitch
CFLAGS += -Wconditional-uninitialized
CFLAGS += -Wreceiver-forward-class
CFLAGS += -Wconversion
CFLAGS += -Wno-sign-conversion
CFLAGS += -Wno-gcc-compat
CFLAGS += -I.

ifeq ($(DEBUG),false)
CFLAGS += -DNDEBUG
CFLAGS += -O3
endif

IOS_SYSROOT ?= $(shell xcodebuild -sdk iphoneos -version Path 2> /dev/null)
CFLAGS_ios := $(CFLAGS)
CFLAGS_ios += -arch arm64
CFLAGS_ios += -DIOS_PLATFORM
CFLAGS_ios += -isysroot $(IOS_SYSROOT)
CFLAGS_ios += -miphoneos-version-min=9

MACOS_SYSROOT ?= $(shell xcodebuild -sdk macosx -version Path 2> /dev/null)
CFLAGS_mac := $(CFLAGS)
CFLAGS_mac += -arch x86_64
CFLAGS_mac += -DMAC_PLATFORM
CFLAGS_mac += -isysroot $(MACOS_SYSROOT)
CFLAGS_mac += -mmacosx-version-min=10.9

CCFLAGS += -std=gnu11

CCFLAGS_ios := $(CFLAGS_ios) $(CCFLAGS)
CCFLAGS_mac := $(CFLAGS_mac) $(CCFLAGS)

CXXFLAGS += -std=gnu++11
CXXFLAGS += -fpermissive
CXXFLAGS += -fno-rtti
CXXFLAGS += -fno-exceptions
CXXFLAGS += -stdlib=libc++

CXXFLAGS_ios := $(CFLAGS_ios) $(CXXFLAGS)
CXXFLAGS_mac := $(CFLAGS_mac) $(CXXFLAGS)

LDFLAGS := -shared
LDFLAGS += -framework Foundation
ifeq ($(DEBUG),true)
LDFLAGS += -g
endif

LDFLAGS_ios := $(LDFLAGS)
LDFLAGS_ios += -arch arm64
LDFLAGS_ios += -isysroot $(IOS_SYSROOT)
LDFLAGS_ios += -miphoneos-version-min=9
LDFLAGS_ios += -framework UIKit

LDFLAGS_mac := $(LDFLAGS)
LDFLAGS_mac += -arch x86_64
LDFLAGS_mac += -isysroot $(MACOS_SYSROOT)
LDFLAGS_mac += -mmacosx-version-min=10.9
LDFLAGS_mac += -framework AppKit

# Source files won't compile these targets if their names contain one or more of these strings
SOURCE_FILTER_ios := /mac
SOURCE_FILTER_mac := /ios

TARGET_ios := $(BUILD_PATH)/bin/ios/libsomething.dylib
TARGET_mac := $(BUILD_PATH)/bin/mac/libsomething.dylib
